@@ -1,31 +1,31 @@
 package Class::PObject::Driver::file;
 
-# $Id: file.pm,v 1.18 2003/09/09 08:46:36 sherzodr Exp $
+# $Id: file.pm,v 1.13 2003/08/27 00:23:37 sherzodr Exp $
 
 use strict;
-#use diagnostics;
 use File::Spec;
 use Log::Agent;
-use Class::PObject::Driver;
-use vars ('$f', '$VERSION', '@ISA');
+use base ('Class::PObject::Driver');
 use Fcntl (':DEFAULT', ':flock', ':mode');
+use vars ('$f', '$VERSION');
 
-@ISA = ("Class::PObject::Driver");
 $VERSION = '2.00';
 $f = 'obj%05d.cpo';
 
 # called when pobject's save() method is called. Note: this is not
 # the same as save() method as the one called by pobject. This is different!
 sub save {
-    my $self = shift;
-    my $class = ref($self) || $self;
-    my ($object_name, $props, $columns) = @_;
-
-    logtrc 3, "%s->save()", $class;
+    my ($self, $object_name, $props, $columns) = @_;
 
     # if 'id' does not already exist, we're being asked to save a newly
     # created object. Before we do that, we create a new id for the object:
-    $columns->{id} ||= $self->generate_id($object_name, $props) or return;
+    if ( defined $columns->{id} ) {
+        logtrc 2, "'id' (%d) already exists. Updating the object data", $columns->{id}
+
+    } else {
+        $columns->{id} = $self->generate_id($object_name, $props) or return;
+        logtrc 2, "'id' was missing, created a new id (%d)", $columns->{id}
+    }
 
     # _filename() returns the name of the file this particular object should
     # be stored in. Look into _filename() for details
@@ -45,7 +45,7 @@ sub save {
     }
 
     # and store frozen data into file:
-    print FH $self->freeze($object_name, $props, $columns);
+    print FH $self->freeze($columns);
     # if we can't close the file handle, it means we couldn't store it.
     unless( close(FH) ) {
         $self->errstr("couldn't save the object: $!");
@@ -53,6 +53,7 @@ sub save {
         return undef
     }
     # if everything went swell, we should return object id
+    logtrc 2, "save() successful. Object id is %d", $columns->{id};
     return $columns->{id}
 }
 
@@ -61,10 +62,9 @@ sub save {
 
 sub load_ids {
     my $self = shift;
-    my $class = ref($self) || $self;
     my ($object_name, $props, $terms, $args) = @_;
 
-    logtrc 3, "%s->load_ids(@_)", $class;
+    logtrc 2, "load_ids(%s, %s, %s, %s)", $object_name, $props, $terms, $args;
 
     # if we come this far, we're being asked to return either all the objects,
     # or by some criteria
@@ -84,16 +84,21 @@ sub load_ids {
         logerr $self->errstr;
         return undef
     }
+
+    logtrc 2, "Browsing '%s'", $object_dir;
+
     my $n = 0;
     while ( my ($filename, $stat) = each %files ) {
         # if 'limit' was given, and 'offset' is missing and sort is not given,
         # then check we have already reached our 'limit'. Otherwise, we need to
         # load all the objects to the memory before we can return the data set
         if ( defined($args->{limit}) && (!$args->{offset}) && (!$args->{'sort'}) && ($n == $args->{limit}) ) {
+            logtrc 2, "only 'limit' (%d) was defined and been met. Exiting the loop now", $args->{limit};
             last
         }
         # if it is a directory, then skip to the next file
         if ( S_ISDIR($stat->mode) ) {
+            logtrc 2, "'%s' is a directory. Skipping to the next", $filename;
             next
         }
 
@@ -104,6 +109,7 @@ sub load_ids {
         $filef_pattern    =~ s/\./\\./g;
 
         unless ( $filename =~ m/^$filef_pattern$/ ) {
+            logtrc 2, "'%s' doesn't match the regex '%s'. Skipping", $filename, $filef_pattern;
             next
         }
         # we open the file with read-only flag
@@ -122,19 +128,24 @@ sub load_ids {
         unless( defined $datastr ) {
             next
         }
-        my $data = $self->thaw($object_name, $props, $datastr);
+        my $data = $self->thaw($datastr);
         if ( $self->_matches_terms($data, $terms) ) {
-            push @data_set, keys %$args ? $data : $data->{id};
+            if ( keys %$args ) {
+                push @data_set, $data;
+            } else {
+                push @data_set, $data->{id}
+            }
             $n++
         }
     }
     untie(%files);
 
     # returning post-processed data set
+    logtrc 2, "%d objects were found matching the 'terms'", scalar(@data_set);
     unless ( keys %$args ) {
         return \@data_set
     }
-
+    
     my $data_set = $self->_filter_by_args(\@data_set, $args);
     return [ map { $_->{id} } @$data_set ]
 }
@@ -154,11 +165,7 @@ sub load_ids {
 
 # load_by_id() is called only while object is to be retrieved by its id
 sub load {
-    my $self = shift;
-    my $class = ref($self) || $self;
-    my ($object_name, $props, $id) = @_;
-
-    logtrc 3, "%s->load(%s)", $class, join ", ", @_;
+    my ($self, $object_name, $props, $id) = @_;
 
     # determine the name of the file for this object
     my $filename = $self->_filename($object_name, $props, $id) or return;
@@ -182,7 +189,7 @@ sub load {
         $self->errstr("object is empty");
         return undef
     }
-    return $self->thaw($object_name, $props, $data_str)
+    return $self->thaw($data_str)
 }
 
 
@@ -202,29 +209,6 @@ sub remove {
 
 
 
-
-
-
-sub drop_datasource {
-    my ($self, $object_name, $props, $i) = @_;
-
-
-    my $dir = $self->_dir($object_name, $props);
-    logtrc 3, "removing '%s'", $dir;
-    require File::Path;
-    unless ( File::Path::rmtree($dir) ) {
-        $self->errstr( "couldn't unlink '%s': %s", $dir, $!);
-        return undef
-    }
-    return 1
-}
-
-
-
-
-
-
-
 sub generate_id {
     my ($self, $object_name, $props) = @_;
 
@@ -263,7 +247,7 @@ sub generate_id {
 sub _filename {
     my ($self, $object_name, $props, $id) = @_;
 
-    unless ( $object_name && defined($id) ) {
+    unless ( $object_name && $id ) {
         logcroak "Usage: _filename(\$id)";
     }
     my $dir = $self->_dir($object_name, $props) or return;
@@ -298,25 +282,33 @@ sub _dir {
             return undef
         }
     }
-    # return the directory
-    return $object_dir
+
+  # return the directory
+  return $object_dir
 }
 
+
+
+
+
+
+
 1;
 
 __END__;
 
+=pod
+
 =head1 NAME
 
 Class::PObject::Driver::file - Default PObject driver
 
 =head1 SYNOPSIS
 
-    pobject Person => {
-        columns   => ['id', 'name', 'email']
-        datasource=> 'data',
-        serializer => 'xml'
-    };
+  pobject Person => {
+    columns   => ['id', 'name', 'email']
+    datasource=> 'data'
+  };
 
 =head1 DESCRIPTION
 
@@ -332,11 +324,6 @@ This data source is a folder in your operating system, inside which objects will
 Pobject will create a folder for each object type inside the I<datasource> folder, and will store
 all the objects of the same type in their own folders.
 
-Other supported property is I<serialiazer>, which defaults to I<storable> if the value is missing.
-This serializer defines the serializing and de-serializing method used by object driver.
-Possible values are I<xml>, which requires L<XML::Dumper|XML::Dumper> to have been installed,
-and I<dumper>, which requires L<Data::Dumper|Data::Dumper> and 'storable'.
-
 =head1 SUPPORTED FEATURES
 
 Class::PObject::Driver::file overrides following methods of Class::PObject::Driver
@@ -391,20 +378,50 @@ C<_filename($self, $pobject_name, \%properties)> is called to get a path to a fi
 object should be stored into. C<_filename()> will call C<_dir()> method to get the object directory,
 and builds a filename inside this directory.
 
+=item *
+
+C<freeze($self, \%data)> and C<thaw($self, $string)> are used to serialize and de-serialize the data
+to make it suitable for storing into disk. By default C<freeze()> and C<thaw()> use
+L<Data::Dumper|Data::Dumper>. If you want to use some other method, you can subclass
+Class::PObject::Driver::file and define your own C<freeze()> and C<thaw()> methods:
+
+    # Inside Class/PObject/Driver/my_file.pm
+    package Class::PObject::Driver::my_file;
+    use base ('Class::PObject::Driver::file');
+    require Storable;
+
+    sub freeze {
+        my ($self, $data) = @_;
+        return Storable::freeze($data)
+    }
+
+    sub thaw {
+        my ($self, $string) = @_;
+        return Storable::thaw($string)
+    }
+
+    1;
+
+    # Inside Article.pm, for example:
+    package Article;
+    pobject {
+        columns => ['id', 'title', 'author', 'content'],
+        driver  => 'my_file'
+    };
+
 =back
 
 =head1 OBJECT STORAGE
 
 Each object is stored as a separate file. File name pattern for each object file is defined in
-C<$Class::PObject::Driver::file::f> global variable, and is C<obj%05.cpo> by default, where C<%05>
-will be replaced with the I<id> of the object, zero-padded if necessary.
+C<$Class::PObject::Driver::file::f> global variable, is is C<obj%05.cpo> by default, where C<%05>
+will be replaced with the I<id> of the object, zeroes padded if necessary.
 
 B<Note:> extension '.cpo' stands for B<C>lass::B<PO>bject.
 
 =head1 SERIALIZATION
 
-Objects are serialized and de-serialized with the help of C<freeze()> and C<thaw()> methods
-provided by its base class, L<Class::PObject::Driver|Class::PObject::Driver>.
+Objects are serialized using standard L<Data::Dumper|Data::Dumper>
 
 =head1 ID GENERATION
 
@@ -416,25 +433,24 @@ the path to this folder) in a file called "counter.cpo".
 
 Removing F<counter.cpo> from the directory will force PObject to reset object ids. This may be a problem
 if there already are objects in the directory, and they may be overridden by new ids. I realize
-this is a scary limitation, which will eventually be addressed.
+this is a scary limitation, which will be eventually addressed.
 
-In the meanwhile, just don't make habit of removing F<counter.cpo> :-).
+In the meanwhile, just don't make habit of removing F<counter.cpo>!
 
 =head1 EFFICIENCY
 
 Since the driver doesn't keep an index of any kind, the most efficient way of loading the data is by its id.
-A relatively simple C<load(undef, {limit=>n})> syntax is also reasonably efficient.
+A relatively simple C<load(undef, {limit=>n})> syntax is also relatively fast.
 
-    $p       = Person->load(451);
-    @people  = Person->load();
-    @group   = Person->load(undef, {limit=>100});
+  my $p       = Person->load(451);
+  my @people  = Person->load();
+  my @group   = Person->load(undef, {limit=>100});
 
 as load() becomes complex, the performance gets degrading:
 
-    @people = Person->load( {name=>"Sherzod"}, 
-                            {sort=>'age', direction=>'desc', limit=>10, offset=>4} );
+  my @people = Person->load({name=>"Sherzod"}, {sort=>'age', direction=>'desc', limit=>10, offset=>4});
 
-To perform the above search, the driver walks through all the objects available in the I<datasource>, pushes all the objects matching 'name="sherzod"' to the data-set, then, just before returning the data set, performs sort, limit and offset calculations.
+To perform the above search, the driver walks through all the objects available in the I<datasource>, pushes all the objects matching 'name="sherzod"' to the data-set, then, just before returning the data set, performs sort, limit and offset calculations. 
 
 As you imagine, as the number of objects in the datasource increases, this operation will become more costly.
 
@@ -443,8 +459,8 @@ As you imagine, as the number of objects in the datasource increases, this opera
 L<Class::PObject>, L<Class::PObject::Driver::mysql>,
 L<Class::PObject::Driver::file>
 
-=head1 COPYRIGHT AND LICENSE
+=head1 AUTHOR
 
-For author and copyright information refer to Class::PObject's L<online manual|Class::PObject>.
+Sherzod Ruzmetov <sherzodr@cpan.org>
 
 =cut
